@@ -2,18 +2,16 @@
 
 Things left to complete:
 
-    - TODO Add sig to ONE Light uuids.
     - TODO Save changes to cache.
     - TODO Fix update cache in AlyxONE - save parquet table.
     - TODO save parquet in update_filesystem.
 """
 import collections.abc
-import concurrent.futures
 import warnings
 import logging
-import os
+import packaging.version
 from datetime import datetime, timedelta
-from functools import lru_cache, reduce
+from functools import lru_cache, partial
 from inspect import unwrap
 from pathlib import Path, PurePosixPath
 from typing import Any, Union, Optional, List
@@ -22,9 +20,9 @@ from uuid import UUID
 import pandas as pd
 import numpy as np
 import requests.exceptions
+
 from iblutil.io import parquet, hashfile
-from iblutil.util import Bunch
-from iblutil.numerical import ismember2d
+from iblutil.util import Bunch, flatten
 
 import one.params
 import one.webclient as wc
@@ -93,7 +91,7 @@ class One(ConversionMixin):
 
     def _load_cache(self, cache_dir=None, **kwargs):
         meta = self._cache['_meta']
-        INDEX_KEY = 'id'
+        INDEX_KEY = '.?id'
         for cache_file in Path(cache_dir or self.cache_dir).glob('*.pqt'):
             table = cache_file.stem
             # we need to keep this part fast enough for transient objects
@@ -106,21 +104,19 @@ class One(ConversionMixin):
             meta['loaded_time'] = datetime.now()
             meta['expired'] |= datetime.now() - created > self.cache_expiry
 
+            # Convert to str ids
+            cache = util.cache_int2str(cache)
+
             # Set the appropriate index if none already set
             if isinstance(cache.index, pd.RangeIndex):
-                num_index = [f'{INDEX_KEY}_{n}' for n in range(2)]
-                try:
-                    int_eids = cache[num_index].any(axis=None)
-                except KeyError:
-                    int_eids = False
-                cache.set_index(num_index if int_eids else INDEX_KEY, inplace=True)
+                idx_columns = cache.filter(regex=INDEX_KEY).columns.tolist()
+                if len(idx_columns) == 0:
+                    raise KeyError('Failed to set index')
+                cache.set_index(idx_columns, inplace=True)
 
             # Check sorted
-            is_sorted = (cache.index.is_monotonic_increasing
-                         if isinstance(cache.index, pd.MultiIndex)
-                         else True)
             # Sorting makes MultiIndex indexing O(N) -> O(1)
-            if table == 'datasets' and not is_sorted:
+            if not cache.index.is_monotonic_increasing:
                 cache.sort_index(inplace=True)
 
             self._cache[table] = cache
@@ -173,19 +169,8 @@ class One(ConversionMixin):
         list of pathlib.Path
             A local file path list
         """
-        out_files = []
-        if hasattr(dsets, 'iterrows'):
-            dsets = list(map(lambda x: x[1], dsets.iterrows()))
-        # Timeout based a download speed of 5 Mb/s
-        timeout = reduce(lambda x, y: x + (y.get('file_size', 0) or 0), dsets, 0) / 625000
-        with concurrent.futures.ThreadPoolExecutor(max_workers=N_THREADS) as executor:
-            # TODO Subclass can just call web client method directly, no need to pass hash, etc.
-            futures = [executor.submit(self._download_dataset, dset, file_size=dset['file_size'],
-                                       hash=dset['hash'], **kwargs) for dset in dsets]
-            concurrent.futures.wait(futures, timeout=np.ceil(timeout) + 10)
-            for future in futures:
-                out_files.append(future.result())
-        return out_files
+        # Looking to entirely remove method
+        pass
 
     def _download_dataset(self, dset, cache_dir=None, **kwargs) -> Path:
         """
@@ -294,13 +279,13 @@ class One(ConversionMixin):
                 query = util.ensure_list(value)
                 datasets = self._cache['datasets']
                 if self._index_type() is int:
-                    isin, _ = ismember2d(datasets[['eid_0', 'eid_1']].values,
-                                         np.array(sessions.index.values.tolist()))
+                    idx = np.array(sessions.index.tolist())
+                    datasets = datasets.loc[(idx[:, 0], idx[:, 1], ), ]
                 else:
-                    isin = datasets['eid'].isin(sessions.index.values)
+                    datasets = datasets.loc[(sessions.index.values, ), :]
                 # For each session check any dataset both contains query and exists
                 mask = (
-                    (datasets[isin]
+                    (datasets
                         .groupby(index, sort=False)
                         .apply(lambda x: all_present(x['rel_path'], query, x['exists'])))
                 )
@@ -313,24 +298,24 @@ class One(ConversionMixin):
         # Return results
         if sessions.size == 0:
             return ([], None) if details else []
+        sessions = sessions.sort_values(['date', 'subject', 'number'], ascending=False)
         eids = sessions.index.to_list()
         if self._index_type() is int:
             eids = parquet.np2str(np.array(eids))
 
         if details:
-            return eids, sessions.reset_index().iloc[:, 2:].to_dict('records', Bunch)
+            return eids, sessions.reset_index(drop=True).to_dict('records', Bunch)
         else:
             return eids
 
-    def _check_filesystem(self, datasets, offline=None, update_exists=True, clobber=False):
+    def _check_filesystem(self, datasets, offline=None, update_exists=True):
         """Update the local filesystem for the given datasets.
 
         Given a set of datasets, check whether records correctly reflect the filesystem.
         Called by load methods, this returns a list of file paths to load and return.
-        TODO This needs changing; overlaod for downloading?
+        TODO This needs changing; overload for downloading?
          This changes datasets frame, calls _update_cache(sessions=None, datasets=None) to
          update and save tables.  Download_datasets can also call this function.
-        TODO Remove clobber
 
         Parameters
         ----------
@@ -341,46 +326,54 @@ class One(ConversionMixin):
             repository
         update_exists : bool
             If true, the cache is updated to reflect the filesystem
-        clobber : bool
-            If true and not offline, datasets are re-downloaded regardless of local filesystem
 
         Returns
         -------
         A list of file paths for the datasets (None elements for non-existent datasets)
         """
-        if offline or self.offline:
-            files = []
-            if isinstance(datasets, pd.Series):
-                datasets = pd.DataFrame([datasets])
-            elif not isinstance(datasets, pd.DataFrame):
-                # Cast set of dicts (i.e. from REST datasets endpoint)
-                datasets = util.datasets2records(list(datasets))
-            for i, rec in datasets.iterrows():
-                file = Path(self.cache_dir, *rec[['session_path', 'rel_path']])
-                if file.exists():
-                    # TODO Factor out; hash & file size also checked in _download_file;
-                    #  see _update_cache - we need to save changed cache
-                    files.append(file)
-                    new_hash = hashfile.md5(file)
-                    new_size = file.stat().st_size
-                    hash_mismatch = rec['hash'] and new_hash != rec['hash']
-                    size_mismatch = rec['file_size'] and new_size != rec['file_size']
-                    # TODO clobber and tag mismatched
-                    if hash_mismatch or size_mismatch:
-                        # the local file hash doesn't match the dataset table cached hash
-                        # datasets.at[i, ['hash', 'file_size']] = new_hash, new_size
-                        # Raise warning if size changed or hash changed and wasn't empty
-                        if size_mismatch or (hash_mismatch and rec['hash']):
-                            _logger.warning('local md5 or size mismatch')
-                else:
-                    files.append(None)
-                if rec['exists'] != file.exists():
-                    datasets.at[i, 'exists'] = not rec['exists']
-                    if update_exists:
-                        self._cache['datasets'].loc[i, 'exists'] = rec['exists']
-        else:
-            # TODO deal with clobber and exists here?
-            files = self._download_datasets(datasets, update_cache=update_exists, clobber=clobber)
+
+        if isinstance(datasets, pd.Series):
+            datasets = pd.DataFrame([datasets])
+        elif not isinstance(datasets, pd.DataFrame):
+            # Cast set of dicts (i.e. from REST datasets endpoint)
+            datasets = util.datasets2records(list(datasets))
+        indices_to_download = []  # indices of datasets that need (re)downloading
+        files = []  # file path list to return
+        # First go through datasets and check if file exists and hash matches
+        for i, rec in datasets.iterrows():
+            file = Path(self.cache_dir, *rec[['session_path', 'rel_path']])
+            if file.exists():
+                # Check if there's a hash mismatch
+                new_hash = hashfile.md5(file)
+                hash_mismatch = rec['hash'] and new_hash != rec['hash']
+                new_size = file.stat().st_size
+                size_mismatch = rec['file_size'] and new_size != rec['file_size']
+                if hash_mismatch or size_mismatch:
+                    full_relative_path = rec.session_path + rec.rel_path
+                    _logger.warning(f'local md5 or size mismatch on dataset: {full_relative_path}')
+                    # Mismatch: add this index to list of datasets that need downloading
+                    indices_to_download.append(i)
+                files.append(file)  # File exists so add to file list
+            else:
+                # File doesn't exist so add None to output file list
+                files.append(None)
+                # Add this index to list of datasets that need downloading
+                indices_to_download.append(i)
+            if rec['exists'] != file.exists():
+                datasets.at[i, 'exists'] = not rec['exists']
+                if update_exists:
+                    self._cache['datasets'].loc[(slice(None), i), 'exists'] = not rec['exists']
+
+        # If online and we have datasets to download, call download_datasets with these datasets
+        if not (offline or self.offline) and indices_to_download:
+            dsets_to_download = datasets.loc[indices_to_download]
+            # Returns list of local file paths and set to variable
+            new_files = self._download_datasets(dsets_to_download, update_cache=update_exists)
+            # Add each downloaded file to the output list of files
+            for i, file in zip(indices_to_download, new_files):
+                files[datasets.index.get_loc(i)] = file
+
+        # Return full list of file paths
         return files
 
     def _index_type(self, table='sessions'):
@@ -403,9 +396,9 @@ class One(ConversionMixin):
         """
         table = self._cache[table] if isinstance(table, str) else table
         idx_0 = table.index.values[0]
-        if len(table.index.names) == 2 and all(isinstance(x, int) for x in idx_0):
+        if len(table.index.names) % 2 == 0 and all(isinstance(x, int) for x in idx_0):
             return int
-        elif len(table.index.names) == 1 and isinstance(idx_0, str):
+        elif all(isinstance(x, str) for x in util.ensure_list(idx_0)):
             return str
         else:
             raise IndexError
@@ -440,8 +433,7 @@ class One(ConversionMixin):
             raise alferr.ALFMultipleObjectsFound(f'Multiple sessions in cache for eid {eid}')
         if not full:
             return det.iloc[0]
-        # to_drop = 'eid' if int_ids else ['eid_0', 'eid_1']
-        # det = det.drop(to_drop, axis=1)
+        # to_drop = 'eid' if int_ids else ['eid_0', 'eid_1']  # .reset_index(to_drop, drop=True)
         column = ['eid_0', 'eid_1'] if int_ids else 'eid'
         return self._cache['datasets'].join(det, on=column, how='right')
 
@@ -522,14 +514,14 @@ class One(ConversionMixin):
         eid = self.to_eid(eid)  # Ensure we have a UUID str list
         if not eid:
             return datasets.iloc[0:0]  # Return empty
-        if self._index_type() is int:
-            eid_num = parquet.str2np(eid)
-            index = ['eid_0', 'eid_1']
-            isin, _ = ismember2d(datasets[index].to_numpy(), eid_num)
-            datasets = datasets[isin]
-        else:
-            session_match = datasets['eid'] == eid
-            datasets = datasets[session_match]
+        try:
+            if self._index_type() is int:
+                eid_num, = parquet.str2np(eid)
+                datasets = datasets.loc[pd.IndexSlice[eid_num, ], :]
+            else:
+                datasets = datasets.loc[(eid,), :]
+        except KeyError:
+            return datasets.iloc[0:0]  # Return empty
 
         datasets = util.filter_datasets(datasets, **filter_args)
         # Return only the relative path
@@ -702,7 +694,8 @@ class One(ConversionMixin):
         query_type : str
             Query cache ('local') or Alyx database ('remote')
         download_only : bool
-            When true the data are downloaded and the file path is returned.
+            When true the data are downloaded and the file path is returned. NB: The order of the
+            file path list is undefined.
         kwargs : dict
             Additional filters for datasets, including namespace and timescale. For full list
             see the one.alf.spec.describe function.
@@ -941,7 +934,6 @@ class One(ConversionMixin):
         # Make list of metadata Bunches out of the table
         records = (present_datasets
                    .reset_index()
-                   .drop(['eid_0', 'eid_1'], axis=1)
                    .to_dict('records', Bunch))
 
         # Ensure result same length as input datasets list
@@ -978,14 +970,17 @@ class One(ConversionMixin):
             dset_id = parquet.str2np(dset_id)
         elif isinstance(dset_id, UUID):
             dset_id = parquet.uuid2np([dset_id]) if int_idx else str(dset_id)
+        elif not int_idx and not isinstance(dset_id, str):
+            dset_id, = parquet.np2str(dset_id)
         try:
             if int_idx:
-                dataset = self._cache['datasets'].loc[dset_id.tolist()].iloc[0]
+                idx = (slice(None), slice(None), *dset_id.tolist())
+                dataset = self._cache['datasets'].loc[idx, :].squeeze()
             else:
-                dataset = self._cache['datasets'].loc[[dset_id]]
+                dataset = self._cache['datasets'].loc[(slice(None), dset_id), :].squeeze()
+            if dataset.empty:
+                raise alferr.ALFObjectNotFound('Dataset not found')
             assert isinstance(dataset, pd.Series) or len(dataset) == 1
-        except KeyError:
-            raise alferr.ALFObjectNotFound('Dataset not found')
         except AssertionError:
             raise alferr.ALFMultipleObjectsFound('Duplicate dataset IDs')
 
@@ -1240,6 +1235,14 @@ class OneAlyx(One):
         try:
             # Determine whether a newer cache is available
             cache_info = self.alyx.get('cache/info', expires=True)
+
+            # Check version compatibility
+            min_version = packaging.version.parse(cache_info.get('min_api_version', '0.0.0'))
+            if packaging.version.parse(one.__version__) < min_version:
+                warnings.warn(f'Newer cache tables require ONE version {min_version} or greater')
+                return
+
+            # Check whether remote cache more recent
             remote_created = datetime.fromisoformat(cache_info['date_created'])
             local_created = cache_meta.get('created_time', None)
             if local_created and (remote_created - local_created) < timedelta(minutes=1):
@@ -1456,6 +1459,20 @@ class OneAlyx(One):
         return (eids, ses) if details else eids
 
     def _download_datasets(self, dsets, **kwargs) -> List[Path]:
+        """
+         Download a single or multitude of datasets if stored on AWS, otherwise calls
+         OneAlyx._download_dataset
+
+         Parameters
+         ----------
+         dset : dict, str, pd.Series
+             A single or multitude of dataset dictionaries
+
+         Returns
+         -------
+         pathlib.Path
+             A local file path or list of paths
+         """
         # If all datasets exist on AWS, download from there.
         try:
             assert 'exists_aws' in dsets and dsets['exists_aws'].all()
@@ -1463,7 +1480,7 @@ class OneAlyx(One):
             return self._download_aws(map(lambda x: x[1], dsets.iterrows()), **kwargs)
         except Exception as ex:
             _logger.debug(ex)
-            return super()._download_datasets(dsets, **kwargs)
+            return self._download_dataset(dsets, **kwargs)
 
     def _download_aws(self, dsets, clobber=False, **kwargs) -> List[Path]:
         # Download datasets from AWS
@@ -1522,14 +1539,72 @@ class OneAlyx(One):
                     out_files[-1] = None
         return out_files
 
-    def _download_dataset(self, dset, cache_dir=None, update_cache=True, **kwargs):
+    def _dset2url(self, dset, update_cache=True):
         """
-        Download a dataset from an Alyx REST dictionary
+        Converts a dataset into a remote HTTP server URL.  The dataset may be one or more of the
+        following: a dict from returned by the sessions endpoint or dataset endpoint, a record
+        from the datasets cache table, or a file path.  If the dataset is from Alyx and cannot be
+        converted to a URL, 'exists' will be set to False in the corresponding entry in the cache
+        table. Unlike record2url, this method can convert dicts and paths to URLs.
 
         Parameters
         ----------
-        dset : dict, str, pd.Series
-            A single dataset dictionary from an Alyx REST query OR URL string
+        dset : dict, str, pd.Series, pd.DataFrame, list
+            A single or multitude of dataset dictionary from an Alyx REST query OR URL string
+
+        Returns
+        -------
+        str
+            The remote URL of the dataset
+        """
+        did = None
+        if isinstance(dset, str) and dset.startswith('http'):
+            url = dset
+        elif isinstance(dset, (str, Path)):
+            url = self.path2url(dset)
+            if not url:
+                _logger.warning(f'Dataset {dset} not found in cache')
+                return
+        elif isinstance(dset, (list, tuple)):
+            dset2url = partial(self._dset2url, update_cache=update_cache)
+            return list(flatten(map(dset2url, dset)))
+        else:
+            # check if dset is dataframe, iterate over rows
+            if hasattr(dset, 'iterrows'):
+                dset2url = partial(self._dset2url, update_cache=update_cache)
+                url = list(map(lambda x: dset2url(x[1]), dset.iterrows()))
+            elif 'data_url' in dset:  # data_dataset_session_related dict
+                url = dset['data_url']
+                did = dset['id']
+            elif 'file_records' not in dset:  # Convert dataset Series to alyx dataset dict
+                url = self.record2url(dset)  # NB: URL will always be returned but may not exist
+                is_int = all(isinstance(x, (int, np.int64)) for x in util.ensure_list(dset.name))
+                did = np.array(dset.name)[-2:] if is_int else util.ensure_list(dset.name)[-1]
+            else:  # from datasets endpoint
+                url = next((fr['data_url'] for fr in dset['file_records']
+                            if fr['data_url'] and fr['exists']), None)
+                did = dset['url'][-36:]
+
+        # Update cache if url not found
+        if did is not None and not url and update_cache:
+            if isinstance(did, str) and self._index_type('datasets') is int:
+                did, = parquet.str2np(did).tolist()
+            elif self._index_type('datasets') is str and not isinstance(did, str):
+                did = parquet.np2str(did)
+            # NB: This will be considerably easier when IndexSlice supports Ellipsis
+            idx = [slice(None)] * int(self._cache['datasets'].index.nlevels / 2)
+            self._cache['datasets'].loc[(*idx, *util.ensure_list(did)), 'exists'] = False
+
+        return url
+
+    def _download_dataset(self, dset, cache_dir=None, update_cache=True, **kwargs) -> List[Path]:
+        """
+        Download a single or multitude of dataset from an Alyx REST dictionary
+
+        Parameters
+        ----------
+        dset : dict, str, pd.Series, pd.DataFrame, list
+            A single or multitude of dataset dictionary from an Alyx REST query OR URL string
         cache_dir : str, pathlib.Path
             The root directory to save the data to (default taken from ONE parameters)
         update_cache : bool
@@ -1537,44 +1612,22 @@ class OneAlyx(One):
 
         Returns
         -------
-        pathlib.Path
-            A local file path
+        pathlib.Path, list
+            A local file path or list of paths
         """
-        if isinstance(dset, str) and dset.startswith('http'):
-            url = dset
-        elif isinstance(dset, (str, Path)):
-            url = self.path2url(dset)
-            if not url:
-                _logger.warning('Dataset not found in cache')
-                return
-        else:
-            if 'data_url' in dset:  # data_dataset_session_related dict
-                url = dset['data_url']
-                did = dset['id']
-            elif 'file_records' not in dset:  # Convert dataset Series to alyx dataset dict
-                url = self.record2url(dset)
-                did = np.array(dset.name)
-            else:  # from datasets endpoint
-                url = next((fr['data_url'] for fr in dset['file_records']
-                            if fr['data_url'] and fr['exists']), None)
-                did = dset['url'][-36:]
-
-        if not url:
-            if 'session' in dset:
-                dset_str = f"{dset['session'][-36:]}: "\
-                           f"{dset.get('collection', '.')}/{dset.get('name', '')}"
-            else:
-                dset_str = f"{dset.get('session_path', '')}/{dset.get('rel_path', '')}"
-            _logger.warning(f"{dset_str} not found")
-            if update_cache:
-                if isinstance(did, str) and self._index_type('datasets') is int:
-                    did = parquet.str2np(did)
-                elif self._index_type('datasets') is str and not isinstance(did, str):
-                    did = parquet.np2str(did)
-                self._cache['datasets'].loc[did, 'exists'] = False
+        cache_dir = cache_dir or self.cache_dir
+        url = self._dset2url(dset, update_cache=update_cache)
+        if url is None:
             return
-        target_dir = Path(cache_dir or self.cache_dir, get_alf_path(url)).parent
-        return self._download_file(url=url, target_dir=target_dir, **kwargs)
+        if isinstance(url, str):
+            target_dir = str(Path(cache_dir, get_alf_path(url)).parent)
+            return self._download_file(url, target_dir, **kwargs)
+        # must be list of URLs
+        valid_urls = list(filter(None, url))
+        target_dir = [str(Path(cache_dir, get_alf_path(x)).parent) for x in valid_urls]
+        files = self._download_file(valid_urls, target_dir, **kwargs)
+        # Return list of file paths or None if we failed to extract URL from dataset
+        return [None if not x else files.pop(0) for x in url]
 
     def _tag_mismatched_file_record(self, url):
         fr = self.alyx.rest('files', 'list',
@@ -1590,68 +1643,90 @@ class OneAlyx(One):
             self.alyx.rest('files', 'partial_update',
                            id=fr[0]['url'][-36:], data={'json': json_field})
 
-    def _download_file(self, url, target_dir,
-                       clobber=False, offline=None, keep_uuid=False, file_size=None, hash=None):
+    def _download_file(self, url, target_dir, keep_uuid=False, file_size=None, hash=None):
         """
-        Downloads a single file from an HTTP webserver.  The webserver in question is set by the
-        AlyxClient object.
+        Downloads a single file or multitude of files from an HTTP webserver.
+        The webserver in question is set by the AlyxClient object.
 
         Parameters
         ----------
-        url : str
+        url : str, list
             An absolute or relative URL for a remote dataset
-        target_dir : str, pathlib.Path
-            The root directory to download file to
-        clobber : bool
-            If true, overwrites local dataset if any
-        offline : bool, None
-            If true, the file path is returned only if the file exists.  No download will take
-            place
+        target_dir : str, list
+            Absolute path of directory to download file to (including alf path)
         keep_uuid : bool
             If true, the UUID is not removed from the file name (default is False)
-        file_size : int
-            The expected file size to compare with downloaded file
-        hash : str
-            The expected file hash to compare with downloaded file
+        file_size : int, list
+            The expected file size or list of file sizes to compare with downloaded file
+        hash : str, list
+            The expected file hash or list of file hashes to compare with downloaded file
 
         Returns
         -------
         pathlib.Path
-            The file path of the downloaded file
+            The file path of the downloaded file or files
+
+        Example
+        -------
+        >>> file_path = OneAlyx._download_file(
+        ...    'https://example.com/data.file.npy', '/home/Downloads/subj/1900-01-01/001/alf')
         """
-        if offline is None:
-            offline = self.mode == 'local'
-        Path(target_dir).mkdir(parents=True, exist_ok=True)
-        local_path = target_dir / os.path.basename(url)
-        if not keep_uuid:
-            local_path = alfio.remove_uuid_file(local_path, dry=True)
-        if Path(local_path).exists():
-            # the local file hash doesn't match the dataset table cached hash
-            hash_mismatch = hash and hashfile.md5(Path(local_path)) != hash
-            file_size_mismatch = file_size and Path(local_path).stat().st_size != file_size
-            if (hash_mismatch or file_size_mismatch) and not offline:
-                clobber = True
-                if not self.alyx.silent:
-                    _logger.warning(f'local md5 or size mismatch, re-downloading {local_path}')
-        # if there is no cached file, download
-        else:
-            clobber = True
-        if clobber and not offline:
-            local_path, md5 = self.alyx.download_file(
-                url, cache_dir=str(target_dir), clobber=clobber, return_md5=True)
-            # TODO If 404 update JSON on Alyx for data record
-            # post download, if there is a mismatch between Alyx and the newly downloaded file size
-            # or hash flag the offending file record in Alyx for database maintenance
-            hash_mismatch = hash and md5 != hash
-            file_size_mismatch = file_size and Path(local_path).stat().st_size != file_size
-            if hash_mismatch or file_size_mismatch:
-                self._tag_mismatched_file_record(url)
-                # TODO Update cache here
+        assert not self.offline
+        # Ensure all target directories exist
+        [Path(x).mkdir(parents=True, exist_ok=True) for x in set(util.ensure_list(target_dir))]
+
+        # download file(s) from url(s), returns file path(s) with UUID
+        local_path, md5 = self.alyx.download_file(url, target_dir=target_dir, return_md5=True)
+
+        # check if url, hash, and file_size are lists
+        if isinstance(url, (tuple, list)):
+            assert (file_size is None) or len(file_size) == len(url)
+            assert (hash is None) or len(hash) == len(url)
+        for args in zip(*map(util.ensure_list, (file_size, md5, hash, local_path, url))):
+            self._check_hash_and_file_size_mismatch(*args)
+
+        # check if we are keeping the uuid on the list of file names
         if keep_uuid:
             return local_path
+
+        # remove uuids from list of file names
+        if isinstance(local_path, (list, tuple)):
+            return [alfio.remove_uuid_file(x) for x in local_path]
         else:
             return alfio.remove_uuid_file(local_path)
 
+    def _check_hash_and_file_size_mismatch(self, file_size, hash, expected_hash, local_path, url):
+        """
+        Check to ensure the hash and file size of a downloaded file matches what is on disk
+
+        Parameters
+        ----------
+        file_size : int
+            The expected file size to compare with downloaded file
+        hash : str
+            The expected file hash to compare with downloaded file
+        local_path: str
+            The path of the downloaded file
+        url : str
+            An absolute or relative URL for a remote dataset
+        """
+        # verify hash size
+        hash = hash or hashfile.md5(local_path)
+        hash_mismatch = hash and hash != expected_hash
+        # verify file size
+        file_size_mismatch = file_size and Path(local_path).stat().st_size != file_size
+        # check if there is a mismatch in hash or file_size
+        if hash_mismatch or file_size_mismatch:
+            # post download, if there is a mismatch between Alyx and the newly downloaded file size
+            # or hash, flag the offending file record in Alyx for database for maintenance
+            hash_mismatch = expected_hash and expected_hash != hash
+            file_size_mismatch = file_size and Path(local_path).stat().st_size != file_size
+            if hash_mismatch or file_size_mismatch:
+                url = url or self.path2url(local_path)
+                _logger.debug(f'Tagging mismatch for {url}')
+                # tag the mismatched file records
+                self._tag_mismatched_file_record(url)
+
     @staticmethod
     def setup(base_url=None, **kwargs):
         """
@@ -1878,7 +1953,7 @@ class OneAlyx(One):
         # Get ID of fist matching dset
         for idx, rel_path in datasets['rel_path'].items():
             if rel_path.endswith(dset_name):
-                return idx
+                return idx[-1]  # (eid, did)
         raise ValueError(f'Dataset {dset_name} not found in cache')
 
     @util.refresh
